@@ -397,6 +397,10 @@ class MySQLParser(SQLParser):
     def union(self, p):
         return Union(left=p.select0, right=p.select1, unique=False)
 
+    # tableau
+    @_('LPAREN select RPAREN')
+    def select(self, p):
+        return p.select
 
     # create table
     @_('CREATE TABLE identifier select')
