@@ -3,7 +3,7 @@ from collections import defaultdict
 from mindsdb_sql.exceptions import PlanningException
 from mindsdb_sql.parser import ast
 from mindsdb_sql.parser.ast import (Select, Identifier, Join, Star, BinaryOperation, Constant, OrderBy,
-                                    BetweenOperation, Union, NullConstant, CreateTable)
+                                    BetweenOperation, Union, NullConstant, CreateTable, Function)
 
 from mindsdb_sql.parser.dialects.mindsdb.latest import Latest
 from mindsdb_sql.planner.steps import (FetchDataframeStep, ProjectStep, JoinStep, ApplyPredictorStep,
@@ -93,6 +93,24 @@ class QueryPlanner():
 
         return self.plan.add_step(self.get_integration_select_step(select))
 
+    def plan_nested_select(self, select):
+
+        # get all predictors
+        query_predictors = []
+
+        def find_predictors(node, is_table, **kwargs):
+            if is_table and isinstance(node, ast.Identifier):
+                if self.is_predictor(node):
+                    query_predictors.append(node)
+
+        utils.query_traversal(select, find_predictors)
+
+        if len(query_predictors) == 0:
+            # if no predictor inside = run as is
+            return self.plan_integration_nested_select(select)
+        else:
+            return self.plan_predictor_nested_select(select)
+
     def plan_integration_nested_select(self, select):
         fetch_df_select = copy.deepcopy(select)
         deepest_select = get_deepest_select(fetch_df_select)
@@ -100,6 +118,43 @@ class QueryPlanner():
         recursively_disambiguate_identifiers(deepest_select, integration_name, table)
         return self.plan.add_step(FetchDataframeStep(integration=integration_name, query=fetch_df_select))
 
+    def plan_predictor_nested_select(self, select):
+        # plan nested select
+
+        if select.limit == 0:
+            # TODO don't run predictor if limit is 0
+            ...
+        select2 = copy.deepcopy(select.from_table)
+        select2.parentheses = False
+        select2.alias = None
+        self.plan_select(select2)
+
+        last_step = self.plan.steps[-1]
+        group_step = self.plan_group(select, last_step)
+        if group_step is not None:
+            self.plan.add_step(group_step)
+            # don't do project step
+
+        else:
+            # do we need projection?
+            if len(select.targets) != 1 or not isinstance(select.targets[0], Star):
+                # remove prefix alias
+                alias = select.from_table.alias
+                if alias is not None:
+                    alias = alias.parts[0]
+                    for t in select.targets:
+                        if isinstance(t, Identifier) and t.parts[0] == alias:
+                            t.parts.pop(0)
+
+                self.plan_project(select, last_step.result, ignore_doubles=True)
+
+        # do we need limit?
+        last_step = self.plan.steps[-1]
+        if select.limit is not None:
+            last_step = self.plan.add_step(LimitOffsetStep(dataframe=last_step.result, limit=select.limit.value))
+        return last_step
+
+
     def plan_select_from_predictor(self, select):
         predictor_namespace, predictor = get_predictor_namespace_and_name_from_identifier(select.from_table, self.default_namespace)
 
@@ -367,15 +422,41 @@ class QueryPlanner():
 
         return self.plan.add_step(JoinStep(left=select_left_step.result, right=select_right_step.result, query=new_join))
 
-    def plan_project(self, query, dataframe):
+    def plan_group(self, query, last_step):
+        # check group
+        funcs = []
+        for t in query.targets:
+            if isinstance(t, Function):
+                funcs.append(t.op.lower())
+        agg_funcs = ['sum', 'min', 'max', 'avg', 'count', 'std']
+
+        if (
+                query.having is not None
+                or query.group_by is not None
+                or set(agg_funcs) & set(funcs)
+        ):
+            # is aggregate
+            group_by_targets = []
+            for t in query.targets:
+                target_copy = copy.deepcopy(t)
+                group_by_targets.append(target_copy)
+            # last_step = self.plan.steps[-1]
+            return GroupByStep(dataframe=last_step.result, columns=query.group_by, targets=group_by_targets)
+
+
+    def plan_project(self, query, dataframe, ignore_doubles=False):
         out_identifiers = []
+
         for target in query.targets:
-            if isinstance(target, Identifier) or isinstance(target, Star) or isinstance(target, Constant):
+            if isinstance(target, Identifier) \
+                    or isinstance(target, Star) \
+                    or isinstance(target, Function) \
+                    or isinstance(target, Constant):
                 out_identifiers.append(target)
             else:
                 new_identifier = Identifier(str(target.to_string(alias=False)), alias=target.alias)
                 out_identifiers.append(new_identifier)
-        return self.plan.add_step(ProjectStep(dataframe=dataframe, columns=out_identifiers))
+        return self.plan.add_step(ProjectStep(dataframe=dataframe, columns=out_identifiers, ignore_doubles=ignore_doubles))
 
     def get_aliased_fields(self, targets):
         # get aliases from select target
@@ -400,15 +481,15 @@ class QueryPlanner():
             query = join_left
 
             if query.from_table.alias is not None:
-                table_alias = query.from_table.alias.parts[0]
+                table_alias = [query.from_table.alias.parts[0]]
             else:
-                table_alias = query.from_table.parts[-1]
+                table_alias = query.from_table.parts
 
             def add_aliases(node, is_table, **kwargs):
                 if not is_table and isinstance(node, Identifier):
                     if len(node.parts) == 1:
                         # add table alias to field
-                        node.parts.insert(0, table_alias)
+                        node.parts = table_alias + node.parts
 
             query_traversal(query.where, add_aliases)
 
@@ -543,7 +624,7 @@ class QueryPlanner():
             else:
                 return self.plan_integration_select(query)
         elif isinstance(from_table, Select):
-            return self.plan_integration_nested_select(query)
+            return self.plan_nested_select(query)
         elif isinstance(from_table, Join):
             return self.plan_join(query, integration=integration)
         else:
